"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const loader_utils_1 = require("loader-utils");
const path_1 = __importDefault(require("path"));
const cli_1 = require("@graphql-codegen/cli");
const gen_dts_1 = __importDefault(require("./gen-dts"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const { writeFile } = fs_1.promises;
const libDir = path_1.default.resolve(__dirname, '..');
const tsxBaseDir = path_1.default.join(libDir, '__generated__');
mkdirp_1.default.sync(tsxBaseDir);
const graphlqCodegenLoader = async function (gqlContent) {
    const options = loader_utils_1.getOptions(this);
    const callback = this.async();
    const { resourcePath: gqlFullPath, rootContext: userDir } = this;
    const gqlRelPath = path_1.default.relative(userDir, gqlFullPath);
    const tsxRelPath = gqlRelPath + `.tsx`;
    const tsxFullPath = path_1.default.join(tsxBaseDir, tsxRelPath);
    const dtsFullPath = `${gqlFullPath}.d.ts`;
    // Pretend .tsx for later loaders.
    // babel-loader at least doesn't respond the .graphql extension.
    this.resourcePath = `${gqlFullPath}.tsx`;
    const [{ content: tsxContent }] = await cli_1.generate(lodash_merge_1.default({
        config: {
            // withHOC: false,  // True by default
            withHooks: true,
        },
    }, options, {
        documents: gqlContent,
        generates: {
            [tsxFullPath]: {
                plugins: [
                    'typescript',
                    'typescript-operations',
                    'typescript-react-apollo',
                ],
            },
        },
    }), true);
    const dtsContent = await gen_dts_1.default(tsxFullPath);
    await writeFile(dtsFullPath, dtsContent);
    callback(undefined, tsxContent);
};
exports.default = graphlqCodegenLoader;
